"""
Service to manipulate project, from the creation to the tunning
"""

import os
import webbrowser
from typing import List

from cookiecutter.exceptions import OutputDirExistsException
from cookiecutter.main import cookiecutter
from git import Repo
# pylint: disable=unused-import
from github import Github
from github.AuthenticatedUser import AuthenticatedUser
# pylint: enable=unused-import
from github.GithubException import GithubException
from github.Organization import Organization
from github.Repository import Repository
from termcolor import cprint
from tqdm import tqdm

from gamma.services.github import GitHubService
from gamma.utils.input_number import input_int
from gamma.utils.yes_or_no import yes_or_no


class ProjectService(object):
    """ Project service """

    github = GitHubService()
    github_client = None  # type: Github
    github_user = None  # type: AuthenticatedUser

    def __init__(self, gitHubService=None):
        if gitHubService is not None:
            self.github = gitHubService

    def _exec_cookiecutter(self, path: str):
        try:
            return cookiecutter(
                "https://"
                + self.github.get_token()
                + "@git.sourceai.io/freier-niels/cookie-python.git",
                output_dir=path,
            )
        except OutputDirExistsException:
            cprint("a project with the same name already exist in the directory", "red")

        return None

    def _fix_new_path(self, path: str) -> str:
        cprint(
            "A new project will be created{}, we will ask you few questions to customize it:".format(
                " in {}".format(path) if path is not None else ""
            ),
            "green",
        )

        if path is None:
            return "."

        return path

    def _get_orgs_where_user_admin(self) -> List[Organization]:
        orgs = self.github_user.get_orgs()
        number_of_org = 0
        orgs_user_is_admin = []
        for org in orgs:
            number_of_org += 1
        cprint(
            "You are member of {} organisation(s), we will analyse where you are allowed to create a project".format(
                number_of_org
            ),
            "green",
        )

        for org in tqdm(orgs, total=number_of_org, unit="org"):
            admins = org.get_members(role="admin")
            for admin in admins:
                if admin.login == self.github_user.login:
                    orgs_user_is_admin.append(org)

        return orgs_user_is_admin

    def _get_orgs_selector_option(self, orgs: List[Organization]) -> List[str]:
        org_options = []

        for org in orgs:
            if org.description is not None and org.description != "":
                org_options.append("{} - {}".format(org.login, org.description))
            else:
                org_options.append(org.login)
        org_options.append("Create a new organization for this project/case")
        org_options.append("{} - Your personnal account".format(self.github_user.login))

        return org_options

    def _create_personal_repo(self, project_name: str) -> Repository:
        cprint(
            "The project will be created in {}/{}".format(
                self.github_user.login, project_name
            ),
            "green",
        )
        is_private = True
        if yes_or_no("do you want set the project as public?"):
            is_private = False
        try:
            return self.github_user.create_repo(project_name, private=is_private)
        except GithubException:
            cprint(
                "{}/{} already exist".format(self.github_user.login, project_name),
                "red",
            )
            return None

    def _create_organisation_repo(self, org_name: str, project_name: str) -> Repository:
        cprint(
            "The project will be created in {}/{}".format(org_name, project_name),
            "green",
        )
        try:
            return self.github_client.get_organization(org_name).create_repo(
                project_name
            )
        except GithubException:
            cprint("{}/{} already exist".format(org_name, project_name), "red")
            return None

    def _push_base_project(self, repo: Repository, origin, remote_repo):
        repo.git.add(".")
        repo.git.commit(message="base version from Gamma template")
        origin.push(refspec="master:master")
        cprint(
            "Your project is pushed on the remote repository: " + remote_repo.html_url,
            "green",
        )

    def _decide_org_for_creation(self, project_name: str, options: List[str]) -> int:
        cprint(
            "Please choose where you want to create the project {}".format(
                project_name
            ),
            "green",
        )
        for i, opt in enumerate(options):
            print("{} - {}".format(i, opt))
        return input_int("indicate the number of the organization you want to use: ")

    def new(self, path: str):
        """ Will create a new project in the selected organization """
        path = self._fix_new_path(path)

        repo_path = self._exec_cookiecutter(path)
        if repo_path is None:
            return
        project_name = os.path.basename(repo_path)

        if yes_or_no("do you want to initilize a git repository for this new project?"):
            repo = Repo.init(repo_path)
            if yes_or_no("do you want to create a repo on github for your project?"):
                self.github_client = self.github.get_github_client()
                self.github_user = self.github_client.get_user()
                remote_repo = None

                orgs_admin = self._get_orgs_where_user_admin()
                options = self._get_orgs_selector_option(orgs_admin)
                index = self._decide_org_for_creation(project_name, options)

                if index == len(options) - 2:
                    # create a new org then ask again to pick one
                    webbrowser.open(
                        "https://git.sourceai.io/organizations/new",
                        new=2,
                        autoraise=True,
                    )
                    input("continue?")
                    orgs_admin = self._get_orgs_where_user_admin()
                    options = self._get_orgs_selector_option(orgs_admin)
                    index = self._decide_org_for_creation(project_name, options)

                if index == len(options) - 1:
                    remote_repo = self._create_personal_repo(project_name)
                else:
                    remote_repo = self._create_organisation_repo(
                        orgs_admin[index].login, project_name
                    )
                if remote_repo is None:
                    return

                cprint(
                    "Your project is now created on: " + remote_repo.html_url, "green"
                )

                origin = repo.create_remote(
                    "origin",
                    url=remote_repo.clone_url.replace(
                        "https://", "https://" + self.github.get_token() + "@"
                    ),
                )
                if yes_or_no("do you want to push the project?"):
                    self._push_base_project(repo, origin, remote_repo)
